@@ -3,12 +3,13 @@
 // #![ allow( dead_code ) ]
 #![ warn( missing_docs ) ]
 #![ warn( missing_debug_implementations ) ]
+// #![ allow( proc_macro_derive_resolution_fallback ) ]
 
 use quote::{ quote };
 use syn::{ DeriveInput };
-use itertools::{ MultiUnzip, process_results };
+use iter_tools::{ Itertools, process_results };
 
-use wproc_macro::*;
+use proc_macro_tools::*;
 
 pub type Result< T > = std::result::Result< T, syn::Error >;
 
@@ -26,7 +27,7 @@ struct FormerField< 'a >
   pub ty : &'a syn::Type,
   pub non_optional_ty : &'a syn::Type,
   pub is_optional : bool,
-  pub type_container_kind : wproc_macro::ContainerKind,
+  pub type_container_kind : proc_macro_tools::ContainerKind,
 }
 
 ///
@@ -89,20 +90,24 @@ impl syn::parse::Parse for AttributeDefault
 
 fn is_optional( ty : &syn::Type ) -> bool
 {
-  wproc_macro::type_rightmost( ty ) == Some( "Option".to_string() )
+  proc_macro_tools::type_rightmost( ty ) == Some( "Option".to_string() )
 }
 
-//
+///
+/// Extract the first parameter of the type if such exist.
+///
 
 fn parameter_internal_first( ty : &syn::Type ) -> Result< &syn::Type >
 {
-  wproc_macro::type_parameters( ty, 0 ..= 0 )
+  proc_macro_tools::type_parameters( ty, 0 ..= 0 )
   .first()
   .map( | e | *e )
   .ok_or_else( || syn_err!( ty, "Expects at least one parameter here:\n  {}", quote!{ #ty } ) )
 }
 
-//
+///
+/// Extract the first and the second parameters of the type if such exist.
+///
 
 fn parameter_internal_first_two( ty : &syn::Type ) -> Result< ( &syn::Type, &syn::Type ) >
 {
@@ -110,7 +115,7 @@ fn parameter_internal_first_two( ty : &syn::Type ) -> Result< ( &syn::Type, &syn
   {
     syn_err!( ty, "Expects at least two parameters here:\n  {}", quote!{ #ty } )
   };
-  let result = wproc_macro::type_parameters( ty, 0 ..= 1 );
+  let result = proc_macro_tools::type_parameters( ty, 0 ..= 1 );
   let mut iter = result.iter();
   Ok
   ((
@@ -119,35 +124,51 @@ fn parameter_internal_first_two( ty : &syn::Type ) -> Result< ( &syn::Type, &syn
   ),)
 }
 
-//
+///
+/// Generate fields for initializer of a struct setting each field to `None`.
+///
+/// # Sample of output
+///
+/// ```compile_fail
+/// int_1 : core::option::Option::None,
+/// string_1 : core::option::Option::None,
+/// int_optional_1 : core::option::Option::None,
+/// ```
+///
 
 #[inline]
-fn field_none_map( field : &FormerField ) -> syn::Field
+fn field_none_map( field : &FormerField ) -> proc_macro2::TokenStream
 {
   let ident = Some( field.ident.clone() );
-  let colon_token = field.colon_token.clone();
   let tokens = quote! { ::core::option::Option::None };
   let ty2 : syn::Type = syn::parse2( tokens ).unwrap();
-  syn::Field
+
+  quote!
   {
-    attrs : vec![],
-    vis : syn::Visibility::Inherited,
-    ident,
-    colon_token,
-    ty : ty2,
+    #ident : #ty2
   }
 }
 
-//
+///
+/// Generate field of the former for a field of the structure
+///
+/// # Sample of output
+///
+/// ```compile_fail
+/// pub int_1 : core::option::Option< i32 >,
+/// pub string_1 : core::option::Option< String >,
+/// pub int_optional_1 :  core::option::Option< i32 >,
+/// pub string_optional_1 : core::option::Option< String >,
+/// ```
+///
 
 #[inline]
-fn field_optional_map( field : &FormerField ) -> syn::Field
+fn field_optional_map( field : &FormerField ) -> proc_macro2::TokenStream
 {
   let ident = Some( field.ident.clone() );
-  let colon_token = field.colon_token.clone();
   let ty = field.ty.clone();
 
-  let tokens = if is_optional( &ty )
+  let ty2 = if is_optional( &ty )
   {
     quote! { #ty }
   }
@@ -156,47 +177,43 @@ fn field_optional_map( field : &FormerField ) -> syn::Field
     quote! { ::core::option::Option< #ty > }
   };
 
-  let ty2 : syn::Type = syn::parse2( tokens ).unwrap();
-  let vis : syn::Visibility = syn::parse2( quote!( pub ) ).unwrap();
-
-  syn::Field
+  quote!
   {
-    attrs : vec![],
-    vis,
-    ident,
-    colon_token,
-    ty : ty2,
+    pub #ident : #ty2
   }
+
 }
 
-//
+///
+/// Generate code converting a field of the former to the field of the structure.
+///
+/// # Sample of output
+///
+/// ```compile_fail
+/// let int_1 = if self.int_1.is_some()
+/// {
+///   self.int_1.take().unwrap()
+/// }
+/// else
+/// {
+///   let val : i32 = Default::default();
+///   val
+/// };
+/// ```
+///
 
 #[inline]
-fn field_form_map( field : &FormerField ) -> Result< syn::Stmt >
+fn field_form_map( field : &FormerField ) -> Result< proc_macro2::TokenStream >
 {
   let ident = field.ident;
   let ty = field.ty;
-
   let mut default = None;
-  // if field.attrs.len() == 1
-  // {
-  //   let ( key, val, _meta ) = attr_pair_single
-  //   (
-  //     field.attrs.first().ok_or_else( || syn_err!( field.ident, "No attr" ) )?
-  //   )?;
-  //   if key != "default".to_string()
-  //   {
-  //     return Err( syn_err!( field.ident, "Unknown attribute key : {}\nDid you mean `default`", key ) );
-  //   }
-  //   default = Some( val );
-  // }
 
   for attr in field.attrs.iter()
   {
     let key_ident = attr.path.get_ident()
     .ok_or_else( || syn_err!( attr, "Expects simple key of an attirbute, but got:\n  {}", quote!{ #attr } ) )?;
     let key_str = format!( "{}", key_ident );
-    // println!( "key_ident : {}", key_ident );
     match key_str.as_ref()
     {
       "default" =>
@@ -211,11 +228,6 @@ fn field_form_map( field : &FormerField ) -> Result< syn::Stmt >
     }
   }
 
-  // if false
-  // {
-  //   default = Some( 13 );
-  // }
-
   let tokens = if field.is_optional
   {
 
@@ -282,11 +294,12 @@ fn field_form_map( field : &FormerField ) -> Result< syn::Stmt >
 
   };
 
-  let stmt : syn::Stmt = syn::parse2( tokens ).unwrap();
-  Ok( stmt )
+  Ok( tokens )
 }
 
-//
+///
+/// Extract name of a field out.
+///
 
 #[inline]
 fn field_name_map( field : &FormerField ) -> syn::Ident
@@ -295,16 +308,30 @@ fn field_name_map( field : &FormerField ) -> syn::Ident
   ident
 }
 
-//
+///
+/// Generate a fomer setter for the field.
+///
+/// # Sample of output
+///
+/// ```compile_fail
+/// pub fn int_1< Src >( mut self, src : Src ) -> Self
+/// where Src : Into< i32 >,
+/// {
+///   debug_assert!( self.int_1.is_none() );
+///   self.int_1 = Some( src.into() );
+///   self
+/// }
+/// ```
+///
 
 #[inline]
-fn field_setter_map( field : &FormerField, former_name_ident : &syn::Ident ) -> Result< syn::Stmt >
+fn field_setter_map( field : &FormerField, former_name_ident : &syn::Ident ) -> Result< proc_macro2::TokenStream >
 {
   let ident = &field.ident;
 
   let tokens = match &field.type_container_kind
   {
-    wproc_macro::ContainerKind::No =>
+    proc_macro_tools::ContainerKind::No =>
     {
       let non_optional_ty = &field.non_optional_ty;
       quote!
@@ -319,7 +346,7 @@ fn field_setter_map( field : &FormerField, former_name_ident : &syn::Ident ) ->
         }
       }
     },
-    wproc_macro::ContainerKind::Vector =>
+    proc_macro_tools::ContainerKind::Vector =>
     {
       let ty = &field.ty;
       let internal_ty = parameter_internal_first( ty )?;
@@ -343,7 +370,7 @@ fn field_setter_map( field : &FormerField, former_name_ident : &syn::Ident ) ->
         }
       }
     },
-    wproc_macro::ContainerKind::HashMap =>
+    proc_macro_tools::ContainerKind::HashMap =>
     {
       let ty = &field.ty;
       let ( k_ty, e_ty ) = parameter_internal_first_two( ty )?;
@@ -368,7 +395,7 @@ fn field_setter_map( field : &FormerField, former_name_ident : &syn::Ident ) ->
         }
       }
     },
-    wproc_macro::ContainerKind::HashSet =>
+    proc_macro_tools::ContainerKind::HashSet =>
     {
       let ty = &field.ty;
       let internal_ty = parameter_internal_first( ty )?;
@@ -394,27 +421,25 @@ fn field_setter_map( field : &FormerField, former_name_ident : &syn::Ident ) ->
     },
   };
 
-  let stmt : syn::Stmt = syn::parse2( tokens )?;
-  Ok( stmt )
-
-  // pub fn int_1< Src >( mut self, src : Src ) -> Self
-  // where Src : Into< i32 >,
-  // {
-  //   debug_assert!( self.int_1.is_none() );
-  //   self.int_1 = Some( src.into() );
-  //   self
-  // }
-
+  Ok( tokens )
 }
 
-//
+///
+/// Generate documentation for the former.
+///
 
-fn doc_generate( name_ident : &syn::Ident ) -> String
+fn doc_generate( name_ident : &syn::Ident ) -> ( String, String )
 {
 
+  let doc_former_mod = format!
+  (
+r#" Implementation of former for [{}].
+"#,
+    name_ident
+  );
+
   let doc_example1 =
 r#"
-// use wtools::former::Former;
 use former::Former;
 #[derive( Former )]
 pub struct Struct1
@@ -424,7 +449,7 @@ pub struct Struct1
 }
 "#;
 
-  let doc = format!
+  let doc_former_struct = format!
   (
 r#" Object to form [{}]. If field's values is not set then default value of the field is set.
 
@@ -436,7 +461,7 @@ For specifing custom default value use attribute `default`. For example:
     name_ident, doc_example1
   );
 
-  doc
+  ( doc_former_mod, doc_former_struct )
 }
 
 //
@@ -456,6 +481,11 @@ pub fn former( input : proc_macro::TokenStream ) -> Result< proc_macro2::TokenSt
   let former_name = format!( "{}Former", name_ident );
   let former_name_ident = syn::Ident::new( &former_name, name_ident.span() );
 
+  // use heck::ToSnakeCase;
+  // let former_snake = name_ident.to_string().to_snake_case();
+  // let former_mod = format!( "{}_former", former_snake );
+  // let former_mod_ident = syn::Ident::new( &former_mod, name_ident.span() );
+
   /* structure attribute */
 
   let mut form_after = quote!
@@ -521,7 +551,7 @@ pub fn former( input : proc_macro::TokenStream ) -> Result< proc_macro2::TokenSt
     let colon_token = &field.colon_token;
     let ty = &field.ty;
     let is_optional = is_optional( &ty );
-    let type_container_kind = wproc_macro::type_container_kind( &ty );
+    let type_container_kind = proc_macro_tools::type_container_kind( &ty );
     let non_optional_ty : &syn::Type = if is_optional { parameter_internal_first( ty )? } else { ty };
     let former_field = FormerField { attrs, vis, ident, colon_token, ty, non_optional_ty, is_optional, type_container_kind };
     Ok( former_field )
@@ -540,7 +570,7 @@ pub fn former( input : proc_macro::TokenStream ) -> Result< proc_macro2::TokenSt
     field_setter_map( &former_field, &former_name_ident ),
   )}).multiunzip();
 
-  let doc = doc_generate( &name_ident );
+  let ( _doc_former_mod, doc_former_struct ) = doc_generate( &name_ident );
   let fields_setter : Vec< _ > = process_results( fields_setter, | iter | iter.collect() )?;
   let fields_form : Vec< _ > = process_results( fields_form, | iter | iter.collect() )?;
 
@@ -562,55 +592,282 @@ pub fn former( input : proc_macro::TokenStream ) -> Result< proc_macro2::TokenSt
       }
     }
 
-    #[doc = #doc]
-    #[derive( Debug )]
-    pub struct #former_name_ident #generics
-    {
-      #(
-        /// Option
-        #fields_optional,
-      )*
-    }
+    // #[doc = #doc_former_mod]
+    // mod #former_mod_ident
+    // {
+      // use;
+      // use super::*;
+      // use super::#name_ident;
+      #[cfg( feature = "in_wtools" )]
+      use wtools::former;
 
-    impl #generics #former_name_ident #generics
-    {
-      ///
-      /// Finish setting options and return formed entity.
-      ///
-      /// If `form_after` defined then associated method is called and its result returned instead of entity.
-      /// For example `form()` of structure with : `#[ form_after( fn after1< 'a >() -> Option< &'a str > )` returns `Option< &'a str >`.
-      ///
-      #[inline]
-      pub fn form #form_generics ( self ) -> #form_after_output
+      #[doc = #doc_former_struct]
+      #[derive( Debug )]
+      pub struct #former_name_ident #generics
       {
-        let result = self._form();
-        #form_after
+        #(
+          /// A field
+          #fields_optional,
+        )*
       }
-      ///
-      /// Finish setting options and return formed entity.
-      ///
-      /// `form_after` has no effect on method `_form` unlike method `form`.
-      ///
-      #[inline]
-      pub fn _form( mut self ) -> #name_ident #generics
+
+      impl #generics #former_name_ident #generics
       {
-        #( #fields_form )*
-        let result = #name_ident
+        ///
+        /// Finish setting options and return formed entity.
+        ///
+        /// If `form_after` defined then associated method is called and its result returned instead of entity.
+        /// For example `form()` of structure with : `#[ form_after( fn after1< 'a >() -> Option< &'a str > )` returns `Option< &'a str >`.
+        ///
+        #[inline]
+        pub fn form #form_generics ( self ) -> #form_after_output
         {
-          #( #fields_names, )*
-        };
-        return result;
+          let result = self._form();
+          #form_after
+        }
+        ///
+        /// Finish setting options and return formed entity.
+        ///
+        /// `form_after` has no effect on method `_form` unlike method `form`.
+        ///
+        #[inline]
+        pub fn _form( mut self ) -> #name_ident #generics
+        {
+          #( #fields_form )*
+          let result = #name_ident
+          {
+            #( #fields_names, )*
+          };
+          return result;
+        }
+        #(
+          /// Field setter.
+          #fields_setter
+        )*
       }
-      #(
-        /// Field setter.
-        #fields_setter
-      )*
-    }
+
+    // }
+    // pub use #former_mod_ident::#former_name_ident;
 
   };
 
-  // println!( "{:#?}", ast );
-  // println!( "{:#?}", result );
-  // let result = proc_macro2::TokenStream::new();
   Ok( result )
 }
+
+//
+// = Input :
+//
+// #[derive( Debug, PartialEq )]
+// pub struct Struct1
+// {
+//   pub int_1 : i32,
+//   string_1 : String,
+//   int_optional_1 : core::option::Option< i32 >,
+//   string_optional_1 : Option< String >,
+//   vec_1 : Vec< String >,
+//   hashmap_strings_1 : std::collections::HashMap< String, String >,
+//   hashset_strings_1 : std::collections::HashSet< String >,
+// }
+
+//
+// = Output :
+//
+// impl Struct1
+// {
+//   pub fn former() -> Struct1Former
+//   {
+//     Struct1Former
+//     {
+//       int_1 : core::option::Option::None,
+//       string_1 : core::option::Option::None,
+//       int_optional_1 : core::option::Option::None,
+//       string_optional_1 : core::option::Option::None,
+//       vec_1 : core::option::Option::None,
+//       hashmap_strings_1 : core::option::Option::None,
+//       hashset_strings_1 : core::option::Option::None,
+//     }
+//   }
+// }
+//
+// //
+//
+// #[derive( Debug )]
+// pub struct Struct1Former
+// {
+//   pub int_1 : core::option::Option< i32 >,
+//   pub string_1 : core::option::Option< String >,
+//   pub int_optional_1 :  core::option::Option< i32 >,
+//   pub string_optional_1 : core::option::Option< String >,
+//   pub vec_1 : core::option::Option< Vec< String > >,
+//   pub hashmap_strings_1 : core::option::Option< std::collections::HashMap< String, String > >,
+//   pub hashset_strings_1 : core::option::Option< std::collections::HashSet< String > >,
+// }
+//
+// //
+//
+// impl Struct1Former
+// {
+//   fn _form( mut self ) -> Struct1
+//   {
+//
+//     let int_1 = if self.int_1.is_some()
+//     {
+//       self.int_1.take().unwrap()
+//     }
+//     else
+//     {
+//       let val : i32 = Default::default();
+//       val
+//     };
+//
+//     let string_1 = if self.string_1.is_some()
+//     {
+//       self.string_1.take().unwrap()
+//     }
+//     else
+//     {
+//       let val : String = Default::default();
+//       val
+//     };
+//
+//     let int_optional_1 = if self.int_optional_1.is_some()
+//     {
+//       Some( self.int_optional_1.take().unwrap() )
+//     }
+//     else
+//     {
+//       None
+//     };
+//
+//     let string_optional_1 = if self.string_optional_1.is_some()
+//     {
+//       Some( self.string_optional_1.take().unwrap() )
+//     }
+//     else
+//     {
+//       None
+//     };
+//
+//     let vec_1 = if self.vec_1.is_some()
+//     {
+//       self.vec_1.take().unwrap()
+//     }
+//     else
+//     {
+//       let val : Vec< String > = Default::default();
+//       val
+//     };
+//
+//     let hashmap_strings_1 = if self.hashmap_strings_1.is_some()
+//     {
+//       self.hashmap_strings_1.take().unwrap()
+//     }
+//     else
+//     {
+//       let val : std::collections::HashMap< String, String > = Default::default();
+//       val
+//     };
+//
+//     let hashset_strings_1 = if self.hashset_strings_1.is_some()
+//     {
+//       self.hashset_strings_1.take().unwrap()
+//     }
+//     else
+//     {
+//       let val : std::collections::HashSet< String > = Default::default();
+//       val
+//     };
+//
+//     Struct1
+//     {
+//       int_1,
+//       string_1,
+//       int_optional_1,
+//       string_optional_1,
+//       vec_1,
+//       hashmap_strings_1,
+//       hashset_strings_1,
+//     }
+//
+//   }
+//
+//   fn form( self ) -> Struct1
+//   {
+//     self._form()
+//   }
+//
+//   pub fn int_1< Src >( mut self, src : Src ) -> Self
+//   where Src : core::convert::Into< i32 >,
+//   {
+//     debug_assert!( self.int_1.is_none() );
+//     self.int_1 = Some( src.into() );
+//     self
+//   }
+//
+//   pub fn string_1< Src >( mut self, src : Src ) -> Self
+//   where Src : core::convert::Into< String >,
+//   {
+//     debug_assert!( self.string_1.is_none() );
+//     self.string_1 = Some( src.into() );
+//     self
+//   }
+//
+//   pub fn string_optional_1< Src >( mut self, src : Src ) -> Self
+//   where Src : core::convert::Into< String >
+//   {
+//     debug_assert!( self.string_optional_1.is_none() );
+//     self.string_optional_1 = Some( src.into() );
+//     self
+//   }
+//
+//   pub fn vec_1( mut self ) -> former::runtime::VectorFormer
+//   <
+//     String,
+//     Vec< String >,
+//     Struct1Former,
+//     impl Fn( &mut Struct1Former, core::option::Option< Vec< String > > )
+//   >
+//   {
+//     let container = self.vec_1.take();
+//     let on_end = | former : &mut Struct1Former, container : core::option::Option< Vec< String > > |
+//     {
+//       former.vec_1 = container;
+//     };
+//     former::runtime::VectorFormer::new( self, container, on_end )
+//   }
+//
+//   pub fn hashmap_strings_1( mut self ) -> former::runtime::HashMapFormer
+//   <
+//     String,
+//     String,
+//     std::collections::HashMap< String, String >,
+//     Struct1Former,
+//     impl Fn( &mut Struct1Former, core::option::Option< std::collections::HashMap< String, String > > )
+//   >
+//   {
+//     let container = self.hashmap_strings_1.take();
+//     let on_end = | former : &mut Struct1Former, container : core::option::Option< std::collections::HashMap< String, String > > |
+//     {
+//       former.hashmap_strings_1 = container;
+//     };
+//     former::runtime::HashMapFormer::new( self, container, on_end )
+//   }
+//
+//   pub fn hashset_strings_1( mut self ) -> former::runtime::HashSetFormer
+//   <
+//     String,
+//     std::collections::HashSet< String >,
+//     Struct1Former,
+//     impl Fn( &mut Struct1Former, core::option::Option< std::collections::HashSet< String > > )
+//   >
+//   {
+//     let container = self.hashset_strings_1.take();
+//     let on_end = | former : &mut Struct1Former, container : core::option::Option< std::collections::HashSet< String > > |
+//     {
+//       former.hashset_strings_1 = container;
+//     };
+//     former::runtime::HashSetFormer::new( self, container, on_end )
+//   }
+//
+// }
+//
\ No newline at end of file
